<?php

namespace Drupal\book_moderation_sync\EventSubscriber;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\content_moderation\ModerationInformationInterface;
use Drupal\book\BookManagerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;

/**
 * Subscriber to sync moderation states between books and their children.
 */
class BookModerationSyncSubscriber implements EventSubscriberInterface {

  protected $entityTypeManager;
  protected $moderationInformation;
  protected $bookManager;
  protected $logger;

  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    ModerationInformationInterface $moderation_information,
    BookManagerInterface $book_manager,
    LoggerChannelFactoryInterface $logger_factory
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->moderationInformation = $moderation_information;
    $this->bookManager = $book_manager;
    $this->logger = $logger_factory->get('book_moderation_sync');
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      'entity.presave' => 'onEntityPresave',
    ];
  }

  /**
   * Handles the entity presave event.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity being saved.
   */
  public function onEntityPresave(EntityInterface $entity) {
    // Check if the entity is a book and is moderated.
    if ($entity->getEntityTypeId() === 'node' && $this->moderationInformation->isModeratedEntity($entity)) {
      $new_state = $entity->get('moderation_state')->value;
      $this->syncModerationState($entity, $new_state);
    }
  }

  /**
   * Synchronizes the moderation state with the book's children.
   *
   * @param \Drupal\Core\Entity\EntityInterface $book
   *   The book entity.
   * @param string $state
   *   The new moderation state.
   */
  protected function syncModerationState(EntityInterface $book, $state) {
    $children = $this->bookManager->getBookChildren($book->id());

    foreach ($children as $child) {
      $child_entity = $this->entityTypeManager->getStorage('node')->load($child['nid']);
      if ($child_entity && $this->moderationInformation->isModeratedEntity($child_entity)) {
        $child_entity->set('moderation_state', $state);
        try {
          $child_entity->save();
        }
        catch (EntityStorageException $e) {
          // Log the error if saving fails.
          $this->logger->error('Failed to sync moderation state for child node @nid: @message', [
            '@nid' => $child_entity->id(),
            '@message' => $e->getMessage(),
          ]);
        }
      }
    }
  }

}